%{
#include "parserincludes.h"
#include "alluvion.yacc.h"

extern YYLTYPE yylloc;

void advance_col (char *yytext)
{
  yylloc.first_column = yylloc.last_column;
  yylloc.last_column += strlen (yytext);
}

#define YY_USER_INIT yylloc.first_line=1;yylloc.first_column=0;yylloc.last_line=1;yylloc.last_column=0;

%}

%x comment

%%
"//".*$ { advance_col (yytext); }
for {
    advance_col (yytext);
    return FOR;
  }
if {
    advance_col (yytext);
    return IF;
  }
else {
    advance_col (yytext);
    return ELSE;
  }
while {
    advance_col (yytext);
    return WHILE;
  }
print {
    advance_col (yytext);
    return PRINT;
  }
"/*" {
    advance_col (yytext);
    BEGIN(comment);
  }
<comment>[^*\n]* { advance_col (yytext); }
<comment>"*"+[^*/\n]+ { advance_col (yytext); }
<comment>"*"+"/" {
    advance_col (yytext);
    BEGIN(INITIAL);
}
[a-zA-Z][a-zA-Z0-9_]* {
    yylval.String = strdup(yytext);
    advance_col (yytext);
    return NAME;
  }
[0-9]+ {
    yylval.Number = atoi (yytext);
    advance_col (yytext);
    return NUM;
  }
'[01 _]*' {
    yylval.String = strdup(yytext);
    advance_col (yytext);
    return BITS;
  }
">=" {
    advance_col (yytext);
    return GE;
  }
"<=" {
    advance_col (yytext);
    return LE;
  }
[ \t]+ { advance_col (yytext); }
<*>[\n] {
    yylloc.last_line ++;
    yylloc.first_line ++;
    yylloc.first_column = 0;
    yylloc.last_column = 0;
  }
. {
    advance_col (yytext);
    return (int) yytext[0];
  }
%%
