// -*-C++-*-

#ifndef ARGUMENTEXPRESSION_H
#define ARGUMENTEXPRESSION_H

#include "expression.h"

class RangesExpression;

typedef enum { BM_MUST_NOT_EXIST, BM_MAY_EXIST, BM_MUST_EXIST } BindMode;

class ArgumentExpression : public Expression
{
public:
  ArgumentExpression (char *pName)
  {
    Name = pName;
    Rng = 0;
  }

  ArgumentExpression (char *pName, YYLTYPE &Loc)
  : Expression (Loc)
  {
    Name = pName;
    Rng = 0;
  }

  ArgumentExpression (char *pName, RangesExpression *pRange)
  {
    Name = pName;
    Rng = pRange;
  }

  ArgumentExpression (char *pName, RangesExpression *pRange, YYLTYPE &Loc)
  : Expression (Loc)
  {
    Name = pName;
    Rng = pRange;
  }

  ~ArgumentExpression ();

  AlluvionType Bind (Function &Fun)
  { return Bind (Fun, AT_NO_TYPE, BM_MUST_EXIST); }

  AlluvionType Bind (Function &Fun, AlluvionType RequiredType, BindMode BM);

  char *GetName () { return Name; }
  RangesExpression *GetRangesExpression () { return Rng; }

  void SetBinding (unsigned pBindVar, AlluvionType pArgType)
  { BindVar = pBindVar;
    ArgType = pArgType;
  }
  unsigned GetBinding () { return BindVar; }

  void Evaluate (Instance &In, Instance &Out);
  int Evaluate (Instance &In);

  void Print (ostream &s) const;

  void PrintState (Instance &I);

private:
  char *Name;
  RangesExpression *Rng;
  unsigned BindVar;
  AlluvionType ArgType;
};

#endif
