#include "expression.h"
#include "instance.h"
#include <iostream>

class BinaryExpression : public Expression
{
public:
  BinaryExpression (char pOperator, Expression *pLeftExpr, Expression *pRightExpr, YYLTYPE &pLoc)
  : Expression (pLoc)
  {
    Operator  = pOperator;
    LeftExpr  = pLeftExpr;
    RightExpr = pRightExpr;
  }

  ~BinaryExpression ()
  { delete LeftExpr;
    delete RightExpr;
  }

  void Print (ostream &s) const
  {
    s << *LeftExpr << Operator << *RightExpr;
  }

  AlluvionType Bind (Function &Fun)
  {
    LeftExpr->BindAndCheck (Fun, AT_CONTROLVAR);
    RightExpr->BindAndCheck (Fun, AT_CONTROLVAR);
    return AT_CONTROLVAR;
  }

  void Evaluate (Instance &In, Instance &Out)
  {
    Out.AccessControlValue () = Evaluate (In);
  }

  int Evaluate (Instance &In)
  {
    int l = LeftExpr->Evaluate (In);
    int r = RightExpr->Evaluate (In);
    switch (Operator)
    {
    case '+': return l + r;
    case '-': return l - r;
    case '*': return l * r;
    case '/': return l / r;
    case '%': return l % r;
    case '^':
      if (r < 0) {} // error!
      int p = 1;
      for (int i = 0; i < r; i ++)
        p *= l;
      return p;
    }
    return 0;
  }

private:
  char Operator;
  Expression *LeftExpr;
  Expression *RightExpr;
};
