#include "boolean.h"
#include "yyltype.h"

class BooleanBinary : public Boolean
{
public:
  BooleanBinary (char pOperator, Boolean *pLeftBool, Boolean *pRightBool, YYLTYPE &pLoc)
  : Boolean (pLoc)
  {
    Operator = pOperator;
    LeftBool = pLeftBool;
    RightBool = pRightBool;
  }

  ~BooleanBinary ()
  { delete LeftBool;
    delete RightBool;
  }

  void Bind (Function &Fun)
  {
    LeftBool->Bind (Fun);
    RightBool->Bind (Fun);
  }

  bool Evaluate (Instance &In)
  {
    bool l = LeftBool->Evaluate (In);
    bool r = RightBool->Evaluate (In);
    switch (Operator)
    {
    case '&': return l && r;
    case '|': return l || r;
    }
    return false;
  }

private:
  char Operator;
  Boolean *LeftBool;
  Boolean *RightBool;
};
