// -*-C++-*-

#include "function.h"
#include "instance.h"

class BuiltInFunction : public Function
{
public:
  BuiltInFunction (char *pName, unsigned InArgCount)
  : Function
     ( pName
     , new ArgumentVector ()
     , new ArgumentVector ()
     , 0
     )
  {
    for (unsigned i = 0; i < InArgCount; i ++)
      InArgs->AddArgument (new ArgumentExpression ("x"));
    InArgs->SetPrimitiveBinding (AT_VAR);
    Vars.resize (InArgCount);
    MaxFanOut = 0;
  }

  virtual void Evaluate (Instance &This, Instance &Out) = 0;

  virtual unsigned GateCount () { return 1; }

  void Report ()
  {
    if (InstanceCount > 0)
      cout << "  " << Name << ": "
           << InstanceCount << " instances, max. fanout = "
           << MaxFanOut << endl;

  }

protected:
  unsigned MaxFanOut;
};

class AndFunction : public BuiltInFunction
{
public:
  AndFunction ()
  : BuiltInFunction ("and", 2)
  {}

  void Evaluate (Instance &This, Instance &Out)
  {
    Value &a = Out.WriteValue (MaxFanOut);

    Value &b = This.GetValue (0); b.Read ();
    Value &c = This.GetValue (1); c.Read ();
    if (b.GetState () == ValueTrue)
    {
      if (c.GetState () == ValueTrue)
      {
        a.SetState (ValueTrue);
        a.SetRealTime (max (b.GetRealTime (), c.GetRealTime ()) + 1);
      }
      else
      {
        a.SetState (ValueFalse);
        a.SetRealTime (c.GetRealTime () + 1);
      }
    }
    else
    {
      a.SetState (ValueFalse);
      if (c.GetState () == ValueTrue)
      {
        a.SetRealTime (b.GetRealTime () + 1);
      }
      else
      {
        a.SetRealTime (min (b.GetRealTime (), c.GetRealTime ()) + 1);
      }
    }

    a.SetMaxTime (max (b.GetMaxTime (), c.GetMaxTime ()) + 1);
  }
};

class OrFunction : public BuiltInFunction
{
public:
  OrFunction ()
  : BuiltInFunction ("or", 2)
  {}

  void Evaluate (Instance &This, Instance &Out)
  {
    Value &a = Out.WriteValue (MaxFanOut);

    Value &b = This.GetValue (0); b.Read ();
    Value &c = This.GetValue (1); c.Read ();
    if (b.GetState () == ValueTrue)
    {
      a.SetState (ValueTrue);
      if (c.GetState () == ValueTrue)
      {
        a.SetRealTime (min (b.GetRealTime (), c.GetRealTime ()) + 1);
      }
      else
      {
        a.SetRealTime (b.GetRealTime () + 1);
      }
    }
    else
    {
      if (c.GetState () == ValueTrue)
      {
        a.SetState (ValueTrue);
        a.SetRealTime (c.GetRealTime () + 1);
      }
      else
      {
        a.SetState (ValueFalse);
        a.SetRealTime (max (b.GetRealTime (), c.GetRealTime ()) + 1);
      }
    }

    a.SetMaxTime (max (b.GetMaxTime (), c.GetMaxTime ()) + 1);
  }
};

class XOrFunction : public BuiltInFunction
{
public:
  XOrFunction ()
  : BuiltInFunction ("xor", 2)
  {}

  void Evaluate (Instance &This, Instance &Out)
  {
    Value &a = Out.WriteValue (MaxFanOut);

    Value &b = This.GetValue (0); b.Read ();
    Value &c = This.GetValue (1); c.Read ();
    if (b.GetState () != c.GetState ())
    {
      a.SetState (ValueTrue);
    }
    else
    {
      a.SetState (ValueFalse);
    }

    a.SetRealTime (max (b.GetRealTime (), c.GetRealTime ()) + 1);
    a.SetMaxTime (max (b.GetMaxTime (), c.GetMaxTime ()) + 1);
  }
};

class NotFunction : public BuiltInFunction
{
public:
  NotFunction ()
  : BuiltInFunction ("not", 1)
  {}

  void Evaluate (Instance &This, Instance &Out)
  {
    Value &a = Out.WriteValue (MaxFanOut);

    Value &b = This.GetValue (0); b.Read ();
    if   (b.GetState () == ValueTrue)
         a.SetState (ValueFalse);
    else a.SetState (ValueTrue);

    a.SetRealTime (b.GetRealTime () + 1);
    a.SetMaxTime (b.GetMaxTime () + 1);
  }
};
