#include "callexpression.h"
#include "program.h"
#include "instance.h"
#include <iostream>
#include "yaccmisc.h"

AlluvionType CallExpression::Bind (Function &Fun)
{
  unsigned Index;
  AlluvionType NT = Fun.CheckName (Name, &Index);
  switch (NT)
  {
  case AT_VAR:
  case AT_VARVECTOR:
  case AT_CONTROLVAR:
  case AT_CONTROLVARVECTOR:
    cerr << "ERROR at " << GetLocation ()
         << ": " << AlluvionTypeName (NT)
         << " \"" << Name
         << "\" used as function\n";
    exit (1);
  case AT_NO_TYPE:
    cerr << "ERROR at " << GetLocation ()
         << ": function \"" << Name
         << "\" does not exist\n";
    exit (1);
  case AT_FUNCTION:
    CalledFun = Fun.GetProgram ()->GetFunction (Index);
  }

  Parameters->BindAndCheck (Fun, AT_VAR);
  if (ControlParameters)
    ControlParameters->BindAndCheck (Fun, AT_CONTROLVAR);

  if (ControlParameters != 0 && CalledFun->GetControlArgs () == 0)
  {
    cerr << "ERROR at " << ControlParameters->GetLocation ()
         << " in function \"" << Fun.GetName ()
         << "\": unneeded control parameters given for function \""
         << CalledFun->GetName () << "\"\n";
    exit (1);
  }

  if (ControlParameters == 0 && CalledFun->GetControlArgs () != 0)
  {
    cerr << "ERROR at " << GetLocation ()
         << " in function \"" << Fun.GetName ()
         << "\": control parameters needed for function \""
         << CalledFun->GetName () << "\"\n";
    exit (1);
  }

  return AT_VAR;
}


void CallExpression::Evaluate (Instance &In, Instance &Out)
{
  try
  {
    Instance (CalledFun).Evaluate (In, Out, Parameters, ControlParameters);
  }
  catch (TooManyArguments)
  {
    cerr << "ERROR at " << GetLocation ()
         << " in function \"" << In.GetFunction ()->GetName ()
         << "\": expression returns too many argument bits for function \""
         << CalledFun->GetName () << "\"\n";
    exit (1);
  }
}

void CallExpression::Print (ostream &s) const
{
  s << Name << ' ' << *Parameters;
}
