#include "boolean.h"
#include "expression.h"

class Comparison : public Boolean
{
public:
  Comparison (char pOperator, Expression *pLeftExpr, Expression *pRightExpr, YYLTYPE &pLoc)
  : Boolean (pLoc)
  {
    Operator = pOperator;
    LeftExpr = pLeftExpr;
    RightExpr = pRightExpr;
  }

  ~Comparison ()
  { delete LeftExpr;
    delete RightExpr;
  }

  void Bind (Function &Fun)
  {
    LeftExpr->Bind (Fun);
    RightExpr->Bind (Fun);
  }

  bool Evaluate (Instance &In)
  {
    int l = LeftExpr->Evaluate (In);
    int r = RightExpr->Evaluate (In);
    switch (Operator)
    {
    case '=': return l == r;
    case '>': return l > r;
    case '<': return l < r;
    case 'g': return l >= r;
    case 'l': return l <= r;
    }
    return false;
  }

private:
  char Operator;
  Expression *LeftExpr;
  Expression *RightExpr;
};
