// -*-C++-*-

#ifndef EXPRESSION_H
#define EXPRESSION_H

#include "value.h"
#include "yyltype.h"
#include "alluviontype.h"
#include <vector>
#include <iostream>
#include "yaccmisc.h"

using namespace std;

#ifndef FUNCTION_DEFINED
class Function;
#endif

#ifndef PROGRAM_DEFINED
class Program;
#endif

#ifndef INSTANCE_DEFINED
class Instance;
#endif

class Expression
{
public:
  Expression (YYLTYPE &pLoc) { Loc = pLoc; }
  Expression () {}

  virtual ~Expression () {}

  virtual void Print (ostream &s = cout) const = 0;

  virtual AlluvionType Bind (Function &Fun) = 0;

  void BindAndCheck (Function &Fun, AlluvionType pAT)
  {
    AlluvionType AT = Bind (Fun);
    if (pAT != AT)
    {
      cerr << "ERROR at " << GetLocation ()
           << ": expected " << AlluvionTypeName (pAT)
           << " type expression but got " << AlluvionTypeName (AT)
           << " type expression ";
      Print (); // TODO: print on cerr
      cerr << endl;
      exit (1);
    }
  }

  virtual void Evaluate (Instance &In, Instance &Out) = 0;

  virtual int Evaluate (Instance &In) = 0;

  void SetLocation (YYLTYPE &pLoc) { Loc = pLoc; }
  YYLTYPE &GetLocation () { return Loc; }

private:
  YYLTYPE Loc;
};

inline ostream &operator<< (ostream &s, Expression const &E)
{ E.Print (s); return s; }

#endif
