#include "function.h"
#include "instance.h"
#include "forstatement.h"
#include "yaccmisc.h"
#include <iostream>

using namespace std;

void ForStatement::Bind (Function &Fun)
{
  AlluvionType ExistType = Fun.CheckName (VarName, &BindVar);
  if (ExistType == AT_NO_TYPE)
    BindVar = Fun.AddNewVariable (VarName, AT_CONTROLVAR);
  else if (ExistType != AT_CONTROLVAR)
  {
    cerr << "ERROR at " << GetLocation ()
         << " in function \"" << Fun.GetName ()
         << "\": loop variable \"" << VarName
         << "\" exists and is not of type control variable\n";
    exit (1);
  }

  Rng->Bind (Fun);
  Seq->Bind (Fun);
}

void ForStatement::Evaluate (Instance &Inst)
{
  int &LoopVal = Inst.GetControlValue (BindVar);
  Range R = Rng->EvaluateRange (Inst);
  for ( LoopVal = R.GetBegin ()
      ;    R.GetStep () > 0 && LoopVal <= R.GetEnd ()
        || R.GetStep () < 0 && LoopVal >= R.GetEnd ()
      ; LoopVal += R.GetStep ()
      )
    Seq->Evaluate (Inst);
}

void ForStatement::Print ()
{
  cout << "  for " << VarName << " = ";
  Rng->Print ();
  cout << "\n  {\n";
  Seq->Print ();
  cout << "  }\n";
}
