#include "program.h"
#include <strings.h>
#include "yaccmisc.h"

void Function::Print ()
{
  OutArgs->Print ();
  cout << " = " << Name << " ";
  InArgs->Print ();
  if (Seq)
  {
    cout << "\n{\n";
    Seq->Print ();
    cout << "}\n";
  }
  else
    cout << " <builtin function>\n";
}


AlluvionType Function::CheckName (char *pName, unsigned *Index)
{
  for (unsigned i = 0; i < Vars.size (); i ++)
    if (strcmp (Vars[i], pName) == 0)
    { if (Index != 0) *Index = i;
      return AT_VAR;
    }

  for (unsigned i = 0; i < VarArrays.size (); i ++)
    if (strcmp (VarArrays[i], pName) == 0)
    { if (Index != 0) *Index = i;
      return AT_VARVECTOR;
    }

  for (unsigned i = 0; i < ControlVars.size (); i ++)
    if (strcmp (ControlVars[i], pName) == 0)
    { if (Index != 0) *Index = i;
      return AT_CONTROLVAR;
    }

  for (unsigned i = 0; i < ControlVarArrays.size (); i ++)
    if (strcmp (ControlVarArrays[i], pName) == 0)
    { if (Index != 0) *Index = i;
      return AT_CONTROLVARVECTOR;
    }

  for (unsigned i = 0; i < Prog->GetFunctionCount (); i ++)
    if (strcmp (Prog->GetFunction (i)->GetName (), pName) == 0)
    { if (Index != 0) *Index = i;
      return AT_FUNCTION;
    }

  return AT_NO_TYPE;
}


void Function::Bind (Program *pProg)
{
  Prog = pProg;

  if (Seq == 0) return; // builtin function

  if (ControlArgs)
    ControlArgs->Bind (*this, AT_CONTROLVAR, BM_MUST_NOT_EXIST);
  InArgs->Bind (*this, AT_VAR, BM_MUST_NOT_EXIST);

  Seq->Bind (*this);

  OutArgs->Bind (*this, AT_VAR, BM_MUST_EXIST);
}

void Function::Report ()
{
  if (InstanceCount > 0)
  {
    cout << "  " << Name << ": "
         << InstanceCount << " instances, "
         << InternalGateCount + ExternalGateCount << " gates ("
         << InternalGateCount << " internal, "
         << ExternalGateCount << " external)\n";
  }
}
