#ifndef FUNCTION_H
#define FUNCTION_H

#include "argumentvector.h"
#include "sequence.h"
#include <vector>
#include "alluviontype.h"
#include "value.h"

using namespace std;

#ifndef PROGRAM_DEFINED
class Program;
#endif

class Function
{
public:
  Function
    ( char *pName
    , ArgumentVector *pOutArgs
    , ArgumentVector *pInArgs
    , Sequence *pSeq
    , ArgumentVector *pControlArgs = 0
    )
  {
    Name = pName;
    OutArgs = pOutArgs;
    InArgs = pInArgs;
    ControlArgs = pControlArgs;
    Seq = pSeq;
    RecursionLevel = 0;
    InstanceCount = 0;
    InternalGateCount = 0;
    ExternalGateCount = 0;
  }

  virtual ~Function ()
  {
    delete InArgs;
    delete OutArgs;
    delete ControlArgs;
    delete Seq;
  }

  char *GetName () { return Name; }
  Sequence *GetSequence () { return Seq; }

  void Print ();

  void Bind (Program *pProg);

  unsigned GetVarCount () { return Vars.size (); }
  unsigned GetVarArrayCount () { return VarArrays.size (); }
  unsigned GetControlVarCount () { return ControlVars.size (); }
  unsigned GetControlVarArrayCount () { return ControlVarArrays.size (); }

  unsigned GetVarArrayIndexDim (unsigned BindVar) { return VarArrayIndexDims[BindVar]; }
  unsigned GetControlVarArrayIndexDim (unsigned BindVar) { return ControlVarArrayIndexDims[BindVar]; }

  unsigned AddNewVariable (char *pName, AlluvionType AT, unsigned IndexDim = 0)
  {
    unsigned Index = 0;
    switch (AT)
    {
    case AT_VAR:
      Index = Vars.size ();
      Vars.push_back (pName);
      break;
    case AT_VARVECTOR:
      Index = VarArrays.size ();
      VarArrays.push_back (pName);
      VarArrayIndexDims.push_back (IndexDim);
      break;
    case AT_CONTROLVAR:
      Index = ControlVars.size ();
      ControlVars.push_back (pName);
      break;
    case AT_CONTROLVARVECTOR:
      Index = ControlVarArrays.size ();
      ControlVarArrays.push_back (pName);
      ControlVarArrayIndexDims.push_back (IndexDim);
      break;
    case AT_FUNCTION:
    case AT_NO_TYPE:
      cerr << "internal error!\n";
      exit (1);
    }
    return Index;
  }

  ArgumentVector *GetOutArgs ()
  { return OutArgs; }

  ArgumentVector *GetInArgs ()
  { return InArgs; }

  ArgumentVector *GetControlArgs ()
  { return ControlArgs; }

  Program *GetProgram () { return Prog; }

  AlluvionType CheckName (char *pName, unsigned *Index = 0);

  void InstanceBegin () { RecursionLevel ++; InstanceCount ++; }
  void InstanceEnd (unsigned pInternalGateCount, unsigned pExternalGateCount)
  {
    InternalGateCount += pInternalGateCount;
    if   (RecursionLevel == 1)
         ExternalGateCount += pExternalGateCount;
    else ExternalGateCount -= pInternalGateCount;
    RecursionLevel --;
  }

  unsigned GetInternalGateCount () { return InternalGateCount; }
  unsigned GetExternalGateCount () { return ExternalGateCount; }

  virtual void Report ();

protected:

  char *Name;
  ArgumentVector *OutArgs;
  ArgumentVector *InArgs;
  ArgumentVector *ControlArgs;
  Sequence *Seq;

  Program *Prog;
  vector<char *> Vars;
  vector<char *> VarArrays;
  vector<unsigned> VarArrayIndexDims;
  vector<char *> ControlVars;
  vector<char *> ControlVarArrays;
  vector<unsigned> ControlVarArrayIndexDims;

  unsigned InstanceCount;
  unsigned RecursionLevel;
  unsigned InternalGateCount;
  int      ExternalGateCount;
};

#define FUNCTION_DEFINED

#endif
