// -*-C++-*-

#include "sequence.h"
#include "boolean.h"

class IfStatement : public Statement
{
public:
  IfStatement (Boolean *pBool, Sequence *pThenSeq, YYLTYPE &pLoc)
  : Statement (pLoc)
  {
    Bool = pBool;
    ThenSeq = pThenSeq;
    ElseSeq = 0;
  }

  IfStatement (Boolean *pBool, Sequence *pThenSeq, Sequence *pElseSeq, YYLTYPE &pLoc)
  : Statement (pLoc)
  {
    Bool = pBool;
    ThenSeq = pThenSeq;
    ElseSeq = pElseSeq;
  }

  ~IfStatement ()
  { delete Bool;
    delete ThenSeq;
    delete ElseSeq;
  }

  void Bind (Function &Fun)
  {
    Bool->Bind (Fun);
    ThenSeq->Bind (Fun);
    if (ElseSeq) ElseSeq->Bind (Fun);
  }

  void Evaluate (Instance &Inst)
  {
    if (Bool->Evaluate (Inst))
      ThenSeq->Evaluate (Inst);
    else
      if (ElseSeq) ElseSeq->Evaluate (Inst);
  }

  void Print () {};

private:
  Boolean *Bool;
  Sequence *ThenSeq;
  Sequence *ElseSeq;
};
