#include "instance.h"
#include "builtinfunction.h"

void Instance::Evaluate (Instance &In, Instance &Out, Expression *Pars, Expression *ControlPars)
{
  Fun->InstanceBegin ();

  if (ControlPars)
  {
    InitArgs (Fun->GetControlArgs ());
    ControlPars->Evaluate (In, *this);

    if (!AtEnd ())
    {
      cerr << "ERROR at " << ControlPars->GetLocation ()
           << " in function \"" << In.GetFunction ()->GetName ()
           << "\": too few control arguments for function \""
           << Fun->GetName () << "\"\n";
      exit (1);
    }
  }

  InitArgs (Fun->GetInArgs ());
  Pars->Evaluate (In, *this);

  if (!AtEnd ())
  {
    cerr << "ERROR at " << Pars->GetLocation ()
         << " in function \"" << In.GetFunction ()->GetName ()
         << "\": too few argument bits for function \""
         << Fun->GetName () << "\"\n";
    exit (1);
  }

  if (Fun->GetSequence ())
  {
    Fun->GetSequence ()->Evaluate (*this);

    InitArgs (Fun->GetOutArgs ());
    while (! AtEnd ())
      Out.WriteValue (ReadValue ());

    In.IncreaseExternalGateCount (InternalGateCount + ExternalGateCount);
  }
  else // builtin function
  {
    ((BuiltInFunction *) Fun)->Evaluate (*this, Out);
    In.IncreaseInternalGateCount (((BuiltInFunction *) Fun)->GateCount ());
  }

  Fun->InstanceEnd (InternalGateCount, ExternalGateCount);
}
