// -*-C++-*-

#ifndef INSTANCE_H
#define INSTANCE_H

#include "function.h"
#include "offsetarray.h"
#include "rangesexpression.h"
#include "vectorexpression.h"
#include "rangeiterator.h"

class TooManyArguments {};

class Instance
{
public:
  Instance (Function *pFun)
  : Vals (pFun->GetVarCount ()),
    ValArrays (pFun->GetVarArrayCount ()),
    ControlVals (pFun->GetControlVarCount ()),
    ControlValArrays (pFun->GetControlVarArrayCount ())
  {
    Fun = pFun;
    InternalGateCount = 0;
    ExternalGateCount = 0;
  }

  // TODO: checken, ob man virtual weglassen kann.
  void Evaluate (Instance &In, Instance &Out, Expression *Pars, Expression *ControlPars);

  Function *GetFunction () { return Fun; }

  Value &GetValue (unsigned Index) { return Vals[Index]; }
  OffsetArray<Value> &GetValueArray (unsigned Index) { return ValArrays[Index]; }
  int &GetControlValue (unsigned Index) { return ControlVals[Index]; }
  OffsetArray<int> &GetControlValueArray (unsigned Index) { return ControlValArrays[Index]; }

  // BEGIN: Argument read-write stuff.

  // Write a completely new value from a built-in function.
  Value &WriteValue (unsigned &MaxReadCount)
  {
    Value &Val = AccessValue ();
    Val.Write (MaxReadCount);
    return Val;
  }

  // Write a copy of an existing value.
  void WriteValue (Value &pVal)
  { AccessValue ().Write (pVal); }

  // Read a value.  If called from a built-in function, call Value.Read ()!
  Value &ReadValue ()
  { return AccessValue (true); }

  int &AccessControlValue ()
  {
    if (!BufferMode)
    {
      int *CVal;
      if (AtEnd ()) throw TooManyArguments ();
      if   (HaveRange)
           CVal = &ControlValArrays[Args->GetArgument (VarIndex)->GetBinding ()][*RI];
      else CVal = &ControlVals[Args->GetArgument (VarIndex)->GetBinding ()];
      Forward ();
      return *CVal;
    }
    else
    {
      BufferControlVals.push_back (0);
      return BufferControlVals.back ();
    }
  }

  bool AtEnd ()
  { return VarIndex >= Args->GetArgumentCount (); }

  void InitArgs (ArgumentVector *pArgs)
  {
    Args = pArgs;
    VarIndex = 0;
    Engage ();
    BufferMode = false;
  }

  // buffered writes to enable (a,b)=(b,a) type assignments

  void InitBufferMode ()
  { BufferMode = true; }

  void FlushBuffer (ArgumentVector *pArgs)
  {
    InitArgs (pArgs);
    for (unsigned i = 0; i < BufferVals.size (); i ++)
      WriteValue (BufferVals[i]);
    for (unsigned i = 0; i < BufferControlVals.size (); i ++)
      AccessControlValue () = BufferControlVals[i];
    BufferVals.clear ();
    BufferControlVals.clear ();
  }

  // END: Argument read/write stuff.

  void IncreaseInternalGateCount (unsigned Inc)
  { InternalGateCount += Inc; }

  void IncreaseExternalGateCount (unsigned Inc)
  { ExternalGateCount += Inc; }

  unsigned GetInternalGateCount () { return InternalGateCount; }
  unsigned GetExternalGateCount () { return ExternalGateCount; }

private:

  Value &AccessValue (bool CheckSet = false)
  {
    Value *Val;

    if (!BufferMode)
    {
      if (AtEnd ()) throw TooManyArguments ();

      if   (HaveRange)
           Val = &ValArrays[Args->GetArgument (VarIndex)->GetBinding ()][*RI];
      else Val = &Vals[Args->GetArgument (VarIndex)->GetBinding ()];

      if (CheckSet && Val->GetState () == ValueNotYet)
      {
        cerr << "ERROR at " << Args->GetArgument (VarIndex)->GetLocation ()
             << " in function \"" << Fun->GetName ()
             << "\": output variable " << Args->GetArgument (VarIndex)->GetName ();
        if (HaveRange) cerr << '[' << RI << ']';
        cerr << " not set\n";
        exit (1);
      }

      Forward ();
    }
    else
    {
      BufferVals.push_back (Value ());
      Val = &BufferVals.back ();
    }

    return *Val;
  }

  void Forward ()
  {
    if (HaveRange)
    { RI ++;
      if (RI.AtEnd ())
      { VarIndex ++;
        Engage ();
      }
    }
    else
    { VarIndex ++;
      Engage ();
    }
  }

  void Engage ()
  {
    if (AtEnd ()) return;
    for (;;)
    {
      RangesExpression *Rng = Args->GetArgument (VarIndex)->GetRangesExpression ();
      HaveRange = (Rng != 0);
      if (HaveRange)
      { RI.SetRanges (Rng->EvaluateRanges (*this));
        if (! RI.AtEnd ()) return;
      }
      else
        return;
      VarIndex ++;
      if (AtEnd ()) return;
    }
  }

  Function *Fun;
  vector<Value> Vals;
  vector<OffsetArray<Value> > ValArrays;
  vector<int> ControlVals;
  vector<OffsetArray<int> > ControlValArrays;
  ArgumentVector *Args;
  unsigned VarIndex;
  RangesIterator RI;
  bool HaveRange;
  bool BufferMode;
  vector<Value> BufferVals;
  vector<int> BufferControlVals;
  unsigned InternalGateCount;
  unsigned ExternalGateCount;
};

#define INSTANCE_DEFINED

#endif
