// -*-C++-*-

#include <vector>
#include "rangesiterator.h"

using namespace std;

template<class Val>
class OffsetArray
{
public:

  OffsetArray () { Array = 0; }

  OffsetArray (unsigned Dim) : lower (Dim), sizes (Dim)
  { Array = 0; }

  ~OffsetArray () { delete[] Array; }

  unsigned Size ()  { return Size (sizes); }

  unsigned Dim () { return sizes.size (); }

  Val &operator[] (vector<int> &p)
  {
    if (lower.size () == 0)
    { lower.resize (p.size ()); sizes.resize (p.size ()); }
    if (!Contains (p))  resize (p);
    return Array[To1D (p)];
  }

  void Print (ostream &s = cout)
  {
    for (RangesIterator RI (lower, sizes); ! RI.AtEnd (); RI ++)
      s << '[' << RI << "] " << Array[To1D (*RI)];
  }

private:

  unsigned To1D (vector<int> &p, vector<int> &plower, vector<int> &psizes)
  {
    int q = (p[0] - plower[0]);
    for (unsigned i = 1; i < p.size (); i ++)
      q = q * psizes[i] + p[i] - plower[i];
    return unsigned (q);
  }

  unsigned To1D (vector<int> &p)
  { return To1D (p, lower, sizes); }

  void resize (vector<int> &newlower, vector<int> &newsizes)
  {
    Val *NewArray = new Val[Size (newsizes)];
    for (RangesIterator RI (lower, sizes); ! RI.AtEnd (); RI ++)
      NewArray[To1D (*RI, newlower, newsizes)] = Array[To1D (*RI)];
    lower = newlower;
    sizes = newsizes;
    delete[] Array;
    Array = NewArray;
  }

  void resize (vector<int> &p)
  {
    vector<int> newlower;
    vector<int> newsizes;
    bool changed = false;
    for (unsigned i = 0; i < lower.size (); i ++)
    {
      int l = lower[i], s = sizes[i], q = p[i];
      if (s == 0) l = q;
      if (q >= l + s)
      { newlower.push_back (l);
        newsizes.push_back (max (s*2, q - l + 1));
        changed = true;
      }
      else if (q < l)
      { int ns = max (s*2, l + s - q + 1);
        newlower.push_back (l+s-ns);
        newsizes.push_back (ns);
        changed = true;
      }
      else
      { newlower.push_back (l);
        newsizes.push_back (s);
      }
    }
    if (changed)  resize (newlower, newsizes);
  }

  bool Contains (vector<int> &p)
  { for (unsigned i = 0; i < lower.size (); i ++)
      if (p[i] < lower[i] || p[i] >= lower[i] + sizes[i])
        return false;
    return true;
  }

  unsigned Size (vector<int> &psizes)
  {
    int s = 1;
    for (unsigned i = 0; i < psizes.size (); i ++)
      s *= psizes[i];
    return unsigned (s);
  }

  Val *Array;
  vector<int> lower;
  vector<int> sizes;
};
