#include "program.h"
#include "builtinfunction.h"
#include "vectorexpression.h"
#include <string.h>

using namespace std;

void Program::CreateBuiltInFunctions ()
{
  AddFunction (new AndFunction);
  AddFunction (new OrFunction);
  AddFunction (new XOrFunction);
  AddFunction (new NotFunction);
}

void Program::Run (char *Call)
{
  while (*Call == ' ' || *Call == '\t' || *Call == '\n') Call ++;

  char *FunStart = Call;
  char *FunEnd = Call;
  while (  (*FunEnd >= 'a' && *FunEnd <= 'z') 
        || (*FunEnd >= 'A' && *FunEnd <= 'Z')
        || (*FunEnd >= '0' && *FunEnd <= '9')
        || *FunEnd == '_'
        )
        FunEnd ++;

  char *FunName = new char[FunEnd-FunStart+1];
  strncpy (FunName, FunStart, FunEnd-FunStart);
  FunName[FunEnd-FunStart] = 0;
  Function *Fun = 0;
  for (unsigned i = 0; i < GetFunctionCount (); i ++)
    if (strcmp (FunName, GetFunction (i)->GetName ()) == 0)
    {
      Fun = GetFunction (i);
      break;
    }
  if (Fun == 0)
  {
    cerr << "ERROR: function \"" << FunName << "\" not defined\n";
    exit (1);
  }

  Instance I (Fun);
  Fun->InstanceBegin ();

  if (Fun->GetControlArgs ())
    I.InitArgs (Fun->GetControlArgs ());
  // TODO: wenn keine control args erforderlich, aber angegeben, dann ...

  Call = FunEnd;
  while (*Call == ' ' || *Call == '\t' || *Call == '\n') Call ++;

  if (*Call == '<')
  {
    while (*Call != '>')
    {
      Call ++;
      char *CPE = Call;
      while (*CPE >= '0' && *CPE <= '9') CPE ++;
      if (CPE == Call)
      {
        cerr << "ERROR: syntax error in program call\n";
        exit (1);
      }
      char tmp = *CPE;
      *CPE = 0;
      I.AccessControlValue () = atoi (Call);
      *CPE = tmp;
      Call = CPE;
      while (*Call == ' ' || *Call == '\t' || *Call == '\n') Call ++;
      if (*Call != ',' && *Call != '>')
      {
        cerr << "ERROR: syntax error in program call\n";
        exit (1);
      }
    }
    Call ++;
    while (*Call == ' ' || *Call == '\t' || *Call == '\n') Call ++;
  }

  if (Fun->GetControlArgs () && !I.AtEnd ())
  {
    cerr << "ERROR: too few control arguments given in program call\n";
    exit (1);
  }

  unsigned InputFanSum = 0; // do we need fanouts of the inputs?
  I.InitArgs (Fun->GetInArgs ());

  if (*Call != '(')
  {
    cerr << "ERROR: syntax error in program call\n";
    exit (1);
  }
  Call ++;
  while (*Call != ')')
  {
    switch (*Call)
    {
    case '0':
      if (I.AtEnd ())
      {
        cerr << "ERROR: too many argument bits given in program call\n";
        exit (1);
      }
      I.WriteValue (InputFanSum).SetState (ValueFalse);
      break;

    case '1':
      if (I.AtEnd ())
      {
        cerr << "ERROR: too many argument bits given in program call\n";
        exit (1);
      }
      I.WriteValue (InputFanSum).SetState (ValueTrue);
      break;

    case ')':
    case ',':
    case '\'':
    case '_':
    case ' ':
    case '\t':
    case '\n':
      break;

    default:
      cerr << "ERROR: syntax error in program call\n";
      exit (1);
    }
    Call ++;
  }
  Call ++;
  while (*Call == ' ' || *Call == '\t' || *Call == '\n') Call ++;
  if (*Call != 0)
  {
    cerr << "ERROR: syntax error in program call\n";
    exit (1);
  }

  if (!I.AtEnd ())
  {
    cerr << "ERROR: too few argument bits given in program call\n";
    exit (1);
  }

  Fun->GetSequence ()->Evaluate (I);

  Fun->InstanceEnd (I.GetInternalGateCount (), I.GetExternalGateCount ());

  Fun->GetOutArgs ()->PrintState (I);

  Report ();

  delete[] FunName;
}
