#ifndef RANGESITERATOR_H
#define RANGESITERATOR_H

#include "range.h"

class RangesIterator
{
public:
  RangesIterator () { RngIsMine = false; }

  RangesIterator (vector<Range> &pRng)
  : Rng (&pRng)
  { for (unsigned i = 0; i < Rng->size (); i ++)
      Indices.push_back ((*Rng)[i].GetEnd ());
    RngIsMine = false;
    CheckInitial ();
  }

  // vector<Range>* will be deleted in destructor!
  RangesIterator (vector<Range> *pRng)
  : Rng (pRng) 
  { for (unsigned i = 0; i < Rng->size (); i ++)
      Indices.push_back ((*Rng)[i].GetEnd ());
    RngIsMine = true;
    CheckInitial ();
  }

  RangesIterator (vector<int> &lower, vector<int> &sizes)
  {
    Rng = new vector<Range>;
    RngIsMine = true;
    for (unsigned i = 0; i < sizes.size (); i ++)
    { Rng->push_back (Range (lower[i], lower[i] + sizes[i] - 1));
      Indices.push_back (lower[i] + sizes[i] - 1);
    }
    CheckInitial ();
  }

  ~RangesIterator ()
  { if (RngIsMine) delete Rng; }

  // vector<Range>* will be deleted in destructor!
  void SetRanges (vector<Range> *pRng)
  {
    if (RngIsMine) delete Rng;
    Rng = pRng;
    RngIsMine = true;
    Indices.resize (Rng->size ());
    for (unsigned i = 0; i < Rng->size (); i ++)
      Indices[i] = (*Rng)[i].GetEnd ();
    CheckInitial ();
  }

  void operator++ (int)
  { unsigned i = Indices.size ();
    do
    {
      i --;
      Indices[i] -= (*Rng)[i].GetStep ();
    }
    while (i > 0 && AtEnd (i));
    if (i > 0 || ! AtEnd (0))
      for (i++; i < (*Rng).size (); i ++)
        Indices[i] = (*Rng)[i].GetEnd ();
  }

  bool AtEnd ()
  { return AtEnd (0); }

  vector<int> &operator* () { return Indices; }

  void Print (ostream &s) const
  {
    for (unsigned i = 0; i < Indices.size (); i ++)
    { if (i != 0) s << ',';
      s << Indices[i];
    }
  }

private:

  bool AtEnd (unsigned i)
  { Range &R = (*Rng)[i];
    return (  R.GetStep () > 0 && Indices[i] < R.GetBegin ()
           || R.GetStep () < 0 && Indices[i] > R.GetBegin ()
           );
  }

  // detect empty ranges and set AtEnd condition
  void CheckInitial ()
  { for (unsigned i = 1; i < Indices.size (); i ++)
      if (AtEnd (i))
        Indices[0] = (*Rng)[0].GetBegin () - (*Rng)[0].GetStep ();
  }

  vector<Range> *Rng;
  bool RngIsMine;
  vector<int> Indices;
};

inline ostream &operator<< (ostream &s, RangesIterator const &RI)
{ RI.Print (s);  return s; }

#endif
