#include "expression.h"
#include "instance.h"
#include <iostream>

class UnaryExpression : public Expression
{
public:
  UnaryExpression (char pOperator, Expression *pExpr, YYLTYPE &pLoc)
  : Expression (pLoc)
  {
    Operator  = pOperator;
    Expr = pExpr;
  }

  ~UnaryExpression ()
  { delete Expr; }

  void Print (ostream &s) const
  {
    s << Operator << *Expr;
  }

  AlluvionType Bind (Function &Fun)
  {
    Expr->BindAndCheck (Fun, AT_CONTROLVAR);
    return AT_CONTROLVAR;
  }

  void Evaluate (Instance &In, Instance &Out)
  {
    Out.AccessControlValue () = Evaluate (In);
  }

  int Evaluate (Instance &In)
  {
    int v = Expr->Evaluate (In);
    switch (Operator)
    {
    case '-': return - v;
    }
    return 0;
  }

private:
  char Operator;
  Expression *Expr;
};
