// -*-C++-*-

#ifndef VALUE_H
#define VALUE_H
#include <iostream>
#include "node.h"

using namespace std;

typedef enum {ValueTrue, ValueFalse, ValueNotYet} ValueState;

class Value
{
public:

  Value ()
  {
    State = ValueNotYet;
    MaxTime = 0;
    RealTime = 0;
    mNode = 0;
  }

  Value (Value const &Val)
  {
    State = Val.GetState ();
    MaxTime = Val.GetMaxTime ();
    RealTime = Val.GetRealTime ();
    mNode = Val.GetNode ();
    if (mNode) mNode->RefBegin ();
  }

  ~Value ()
  {
    if (mNode)
    {
      mNode->RefEnd ();
      if (mNode->GetRefCount () == 0)
         delete mNode;
    }
  }

  Value &operator= (Value const &Val)
  {
    if (mNode)
    {
      mNode->RefEnd ();
      if (mNode->GetRefCount () == 0)
         delete mNode;
    }
    State = Val.GetState ();
    MaxTime = Val.GetMaxTime ();
    RealTime = Val.GetRealTime ();
    mNode = Val.GetNode ();
    if (mNode) mNode->RefBegin ();
    return *this;
  }

  Node *GetNode () const { return mNode; }

  void Write (unsigned &pMaxReadCount)
  {
    if (mNode)
    {
      mNode->RefEnd ();
      if (mNode->GetRefCount () > 0)
        mNode = new Node;
    }
    else
      mNode = new Node;

    mNode->Init (pMaxReadCount);
  }

  void Write (const Value &V)
  {
    if (&V == this) return;  // whoa! self-write would fuck up!

    if (mNode)
    {
      mNode->RefEnd ();
      if (mNode->GetRefCount () == 0)
        delete mNode;
    }

    State    = V.GetState ();
    MaxTime  = V.GetMaxTime ();
    RealTime = V.GetRealTime ();
    mNode    = V.GetNode ();
    if (mNode)  mNode->RefBegin ();
  }

  void Read () { mNode->Read (); }

  ValueState GetState  () const { return State; }
  unsigned GetMaxTime  () const { return MaxTime; }
  unsigned GetRealTime () const { return RealTime; }

  void SetState    (ValueState pState)  { State = pState; }
  void SetMaxTime  (unsigned pMaxTime)  { MaxTime = pMaxTime; }
  void SetRealTime (unsigned pRealTime) { RealTime = pRealTime; }

  void Print (ostream &s = cout)
  {
    switch (State)
    {
    case ValueTrue:
      s << '1';
      break;
    case ValueFalse:
      s << '0';
      break;
    case ValueNotYet:
      s << '-';
      break;
    }
    cout << "\t(" << MaxTime << ", " << RealTime << ")\n";
  }

private:

  ValueState State;
  unsigned MaxTime;
  unsigned RealTime;
  Node *mNode;
};

inline ostream &operator<< (ostream &s, Value &v)
{ v.Print (s);
  return s;
}

#endif
