// -*-C++-*-

#ifndef VECTOREXPRESSION_H
#define VECTOREXPRESSION_H

#include "expression.h"
#include <vector>

using namespace std;

class VectorExpression : public Expression
{
public:
  VectorExpression () {}

  ~VectorExpression ()
  { for (unsigned i = 0; i < Exprs.size (); i ++) delete Exprs[i]; }

  void AddExpression (Expression *Expr)
  { Exprs.push_back (Expr); }

  AlluvionType Bind (Function &Fun)
  {
    if (Exprs.size () == 0)
      return AT_NO_TYPE;

    AlluvionType VAT = Exprs[0]->Bind (Fun);

    for (unsigned i = 1; i < Exprs.size (); i ++)
    {
      AlluvionType AT = Exprs[i]->Bind (Fun);
      if (AT != VAT)
      {
        cerr << "ERROR at " << Exprs[i]->GetLocation ()
             << ": mixed (" << AlluvionTypeName (VAT)
             << ", " << AlluvionTypeName (AT)
             << ") type vector expression not allowed\n";
        exit (1);
      }
    }

    return VAT;
  }

  void Evaluate (Instance &In, Instance &Out)
  {
    for (unsigned i = 0; i < Exprs.size (); i ++)
      Exprs[i]->Evaluate (In, Out);
  }

  int Evaluate (Instance &In)
  {
    return Exprs[0]->Evaluate (In);
  }

  void Print (ostream &s) const
  {
    s << '(';
    for (unsigned i = 0; i < Exprs.size (); i ++)
    {
      if (i > 0) s << ", ";
      Exprs[i]->Print (s);
    }
    s << ')';
  }

private:
  vector<Expression *> Exprs;
};

#endif
